"""
request dispatcher:
data persisting across requests should go here
"""

import os
import pkginfo
import shutil
import tempfile
from webob import Request, Response, exc

here = os.path.dirname(os.path.abspath(__file__))

class SimPyPI(object):

    def __init__(self, directory, index=None):

        self.directory = directory
        assert os.path.exists(directory)

        # request handlers
        self.handlers = dict([(method, getattr(self, method))
                              for method in ('GET', 'POST')])
        # TODO: HEAD, OPTIONS, maybe more

        # cache index HTML
        self.index = index or os.path.join(here, 'templates', 'index.html')
        assert os.path.exists(self.index)
        self.index = file(self.index).read()

    def __call__(self, environ, start_response):

        # get a request object
        request = Request(environ)

        # match the request to a handler
        handler = self.handlers.get(request.method)
        if handler:
            res = handler(request)
        else:
            res = exc.HTTPNotFound()

        return res(environ, start_response)

    def GET(self, request):
        return Response(body=self.index, content_type='text/html')

    def POST(self, request):
        """handle posting a package"""

        # get the package
        try:
            package = request.POST['package']
        except KeyError:
            # sanity check: does the field exist?
            return exc.HTTPBadRequest()

        # sanity check: is it a file? (TODO)
        if not hasattr(package, 'file') or not hasattr(package, 'filename'):
            return exc.HTTPBadRequest()

        # successful response: redirect to the main page
        response = exc.HTTPSeeOther()

        # make a temporary copy for pkginfo
        # (or PaInt?)
        tmpdir = tempfile.mkdtemp()
        try:
            path = os.path.join(tmpdir, package.filename)
            f = file(path, 'w')
            f.write(package.file.read())
            f.close()

            # get package data
            sdist = pkginfo.sdist.SDist(path)

            # put the package in the right place
            self.add_package(sdist)
        except BaseException, e:
            # something bad happen
            response = exc.HTTPBadRequest()
        finally:
            # cleanup
            shutil.rmtree(tmpdir)

        return response

    ### API

    def add_package(self, sdist, move=True):
        """
        add a package to the directory
        """

        # make a directory for the package
        directory = os.path.join(self.directory, sdist.name)
        if not os.path.exists(directory):
            os.mkdir(directory)
        assert os.path.isdir(directory)

        # determine the extension (XXX hacky)
        extensions = ('.tar.gz', '.zip', '.tar.bz2')
        for ext in extensions:
            if sdist.filename.endswith(ext):
                break
        else:
            raise Exception("Extension %s not found: %s" % (extensions, sdist.filename))

        # get the filename destination
        filename = '%s-%s%s' % (sdist.name, sdist.version, ext)
        filename = os.path.join(directory, filename)

        if move:
            # move the file
            shutil.move(sdist.filename, filename)
        else:
            # copy the file
            shutil.copy(sdist.filename, filename)



#!/usr/bin/env python

"""
WSGI factories for simpypi
"""

import optparse
import os
import shutil
import sys
import tempfile

from fileserver import DirectoryServer
from webob import exc
from wsgi import SimPyPI
from wsgiref import simple_server

class NamespacedFileserver(DirectoryServer):

    def __init__(self, app, directory, namespace):
        DirectoryServer.__init__(self, directory)
        self.app = app
        self.namespace = namespace

    def __call__(self, environ, start_response):
        path = environ['PATH_INFO']
        if path == self.namespace:
            response = exc.HTTPMovedPermanently(add_slash=True)
            return response(environ, start_response)
            environ['PATH_INFO'] = '/'
            return DirectoryServer.__call__(self, environ, start_response)
        elif path.startswith(self.namespace + '/'):
            environ['PATH_INFO'] = path[len(self.namespace):]
            return DirectoryServer.__call__(self, environ, start_response)
        return self.app(environ, start_response)


def factory(**app_conf):
    """create a webob view and wrap it in middleware"""
    directory = app_conf['directory']
    app = SimPyPI(**app_conf)
    return NamespacedFileserver(app, directory, '/index')

def main(args=sys.argv[1:]):

    # parse command line options
    usage = '%prog [options]'
    parser = optparse.OptionParser(usage=usage)
    parser.add_option('-p', '--port', dest='port',
                      type='int', default=8080,
                      help="port to run the server on")
    parser.add_option('-d', '--directory', dest='directory',
                      help='directory to serve')
    options, args = parser.parse_args(args)

    # create a temporary directory, if none specified
    tmpdir = None
    if not options.directory:
        tmpdir = tempfile.mkdtemp()
        options.directory = tmpdir

    # serve
    print "http://localhost:%d/" % options.port
    try:
        app = factory(directory=options.directory)
        server = simple_server.make_server(host='0.0.0.0', port=options.port, app=app)
        server.serve_forever()
    finally:
        if tmpdir:
            shutil.rmtree(tmpdir)

if __name__ == '__main__':
    main()



from setuptools import setup, find_packages

try:
    description = file('README.txt').read()
except IOError:
    description = ''

version = "0.1.1"

setup(name='simpypi',
      version=version,
      description="Simple pypi package",
      long_description=description,
      classifiers=[], # Get strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      author='Jeff Hammel',
      author_email='jhammel@mozilla.com',
      url='http://k0s.org/mozilla/hg/simpypi',
      license="MPL",
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          # -*- Extra requirements: -*-
         'WebOb',
         'FileServer >= 0.2.1'
      ],
      entry_points="""
      # -*- Entry points: -*-
      [console_scripts]
      simpypi = simpypi.factory:main
      """,
      )
